// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ChargedFinalState.hh"
#include "Rivet/Particle.hh"
#include "Rivet/RivetBoost.hh"
#include "Rivet/ParticleName.hh"

//#include "HepPDT/defs.h"
#include <fstream>

#include <cstdlib>
#include <string>

//#include "HepPDT/TableBuilder.hh"
//#include "HepPDT/ParticleDataTable.hh"
//#include "HepPDT/TempParticleData.hh"

namespace Rivet {


  class LHCB_2011_NEW : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructor
    LHCB_2011_NEW()
      : Analysis("LHCB_2011_NEW")
    {
      setBeams(PROTON, PROTON);
      setNeedsCrossSection(true);
      _sumPassed25_20 = 0;
      _sumPassed20_25 = 0;
      _sumPassed25_30 = 0;
      _sumPassed30_35 = 0;
      _sumPassed35_40 = 0;
      _sumPassed40_45 = 0;
      _sumPassed20_45 = 0;
      _sumPassed45_45 = 0;
      FSP = 0;
    } 

    //@}


  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

        /// @todo Initialise and register projections here

        addProjection(ChargedFinalState(), "CFS");
    
        /// @todo Book histograms here, e.g.:
      
        _hist_nch_eta25_20 = bookHistogram1D("d01-x01-y01", "Charged Multiplicity (-2.5 < eta < -2.0)", "Nch", "Freq");
        _hist_nch_eta20_25 = bookHistogram1D("d02-x01-y01", "Charged Multiplicity (2.0  < eta <  2.5)", "Nch", "Freq");    
        _hist_nch_eta25_30 = bookHistogram1D("d03-x01-y01", "Charged Multiplicity (2.5  < eta <  3.0)", "Nch", "Freq");
        _hist_nch_eta30_35 = bookHistogram1D("d04-x01-y01", "Charged Multiplicity (3.0  < eta <  3.5)", "Nch", "Freq");
        _hist_nch_eta35_40 = bookHistogram1D("d05-x01-y01", "Charged Multiplicity (3.5  < eta <  4.0)", "Nch", "Freq");
        _hist_nch_eta40_45 = bookHistogram1D("d06-x01-y01", "Charged Multiplicity (4.0  < eta <  4.5)", "Nch", "Freq");
        _hist_nch_eta20_45 = bookHistogram1D("d07-x01-y01", "Charged Multiplicity (2.0  < eta <  4.5)", "Nch", "Freq");
        _hist_eta_nch      = bookHistogram1D("d08-x01-y01", 100, -5.0, 5.0);
    }


    /// Perform the per-event analysis
    void analyze(const Event& event) {

        const double weight = event.weight();
        const ChargedFinalState& cfs = applyProjection<ChargedFinalState>(event, "CFS");
    
        if (cfs.size() < 1) vetoEvent;

        long _multiplicity25_20 = 0;
        long _multiplicity20_25 = 0;
        long _multiplicity25_30 = 0;
        long _multiplicity30_35 = 0;
        long _multiplicity35_40 = 0;
        long _multiplicity40_45 = 0;
        long _multiplicity20_45 = 0;
        long _multiplicity45_45 = 0;
        double _arrayeta[1000];

        //particle loop
        foreach (const Particle& p, cfs.particles()) {

            const PdgId id = fabs(p.pdgId());
            FSP++;

            //select the correct particle types
            bool particleType = 0;
            if (id == 11)    particleType = 1;  //electron
            if (id == 13)    particleType = 1;  //muon
            if (id == 211)   particleType = 1;  //pion
            if (id == 321)    particleType = 1;  //K  
            if (id == 2212)  particleType = 1;  //Proton
       
            //if (particleType == 0) cout << "not right type of particle" << id  << endl;
            //if ( p.genParticle().status() != 1) cout << "Status ID not 1 :" << p.genParticle().status() << endl;          

            if (particleType == 1){
                
                double y = p.momentum().eta();

                if (y > -2.5 && y < -2.0) _multiplicity25_20 ++;
                if (y >  2.0 && y <  2.5) _multiplicity20_25 ++;
                if (y >  2.5 && y <  3.0) _multiplicity25_30 ++;
                if (y >  3.0 && y <  3.5) _multiplicity30_35 ++;
                if (y >  3.5 && y <  4.0) _multiplicity35_40 ++;
                if (y >  4.0 && y <  4.5) _multiplicity40_45 ++;
                if (y >  2.0 && y <  4.5) _multiplicity20_45 ++;
                if (y > -4.5 && y <  4.5){ _arrayeta[_multiplicity45_45] = y;  _multiplicity45_45 ++; }
            }
        }

        //fill muliplicity plots
        if (_multiplicity25_20 > 0) { _sumPassed25_20 += weight; _hist_nch_eta25_20->fill(_multiplicity25_20, weight); }
        if (_multiplicity20_25 > 0) { _sumPassed20_25 += weight; _hist_nch_eta20_25->fill(_multiplicity20_25, weight); }
        if (_multiplicity25_30 > 0) { _sumPassed25_30 += weight; _hist_nch_eta25_30->fill(_multiplicity25_30, weight); }
        if (_multiplicity30_35 > 0) { _sumPassed30_35 += weight; _hist_nch_eta30_35->fill(_multiplicity30_35, weight); }
        if (_multiplicity35_40 > 0) { _sumPassed35_40 += weight; _hist_nch_eta35_40->fill(_multiplicity35_40, weight); }
        if (_multiplicity40_45 > 0) { _sumPassed40_45 += weight; _hist_nch_eta40_45->fill(_multiplicity40_45, weight); }
        if (_multiplicity20_45 > 0) { _sumPassed20_45 += weight; _hist_nch_eta20_45->fill(_multiplicity20_45, weight); }
        
        //fill eta plot
        if (_multiplicity20_45 > 0) {     
            for (int i=0; i <= _multiplicity45_45; i++) _hist_eta_nch->fill(_arrayeta[i], weight);
        }
    }


    /// Normalise histograms etc., after the run
    void finalize() {

      /// @todo Normalise, scale and otherwise manipulate histograms here
      // scale(_h_YYYY, crossSection()/sumOfWeights()); # norm to cross section

      scale(_hist_nch_eta25_20, 1.0/_sumPassed25_20);
      scale(_hist_nch_eta20_25, 1.0/_sumPassed20_25);
      scale(_hist_nch_eta25_30, 1.0/_sumPassed25_30);
      scale(_hist_nch_eta30_35, 1.0/_sumPassed30_35);
      scale(_hist_nch_eta35_40, 1.0/_sumPassed35_40);
      scale(_hist_nch_eta40_45, 1.0/_sumPassed40_45);
      scale(_hist_nch_eta20_45, 1.0/_sumPassed20_45);
      scale(_hist_eta_nch,      1.0/_sumPassed20_45);
      
      cout << "Number of FSP passed to Rivet:" << FSP << endl;
    }

    //@}


  private:



  private:

   
      double _sumPassed25_20;
      double _sumPassed20_25;
      double _sumPassed25_30;
      double _sumPassed30_35;
      double _sumPassed35_40;
      double _sumPassed40_45;
      double _sumPassed20_45;
      double _sumPassed45_45;
      long FSP;



    /// @name Histograms
    //@{

    AIDA::IHistogram1D* _hist_nch_eta25_20;
    AIDA::IHistogram1D* _hist_nch_eta20_25;
    AIDA::IHistogram1D* _hist_nch_eta25_30;
    AIDA::IHistogram1D* _hist_nch_eta30_35;
    AIDA::IHistogram1D* _hist_nch_eta35_40;
    AIDA::IHistogram1D* _hist_nch_eta40_45;
    AIDA::IHistogram1D* _hist_nch_eta20_45;
    AIDA::IHistogram1D* _hist_eta_nch;
    //@}


  };



  // This global object acts as a hook for the plugin system
  AnalysisBuilder<LHCB_2011_NEW> plugin_LHCB_2011_NEW;


}



                /*
                GenVertex* prodVtx = 0;
 
                prodVtx = p.genParticle().production_vertex();
                foreach (const GenParticle* ancestor, particles(prodVtx, HepMC::ancestors)) {

                    if (ancestor->status() == 2){
                        //Find lifetime of mothers here and save as lifetime
                        //double lifetime = datacol.particle( HepPDT::ParticleID(ancestor->pdg_id()))->clifetime();
                        //cout << "pdg ID " << ancestor->pdg_id() << endl;
                        //cout << "lifetime " << lifetime << endl;
                        //mothers_lifetime =+ lifetime;
                    }

                    double lifetime = 5.0;
                } 
                */


        /***************************************************************************************************************
        ****************************************************************************************************************   

const char infile[] = "/afs/cern.ch/sw/lcg/external/HepPDT/3.04.01/x86_64-slc5-gcc43-opt/data/particle.tbl";
        std::ifstream pdfile( infile );
        if( !pdfile ) { 
            std::cerr << "cannot open " << infile << std::endl;
            exit(-1);
        }
        




        HepPDT::ParticleDataTable datacol( "Generic Particle Table" );
 

        {
            // Construct table builder
            HepPDT::TableBuilder  tb(datacol);
            // read the input - put as many here as you want
            // bool  addParticleTable( std::istream&, TableBuilder&,
            //                         bool validate = false );
            // where:  validate=true => verify that the ParticleID is valid
            if( !addParticleTable( pdfile, tb, true ) ) { 
                std::cout << "error reading PDG pdt file " << std::endl; 
            }
         }   // the tb destructor fills datacol


         ****************************************************************************************************************
         ****************************************************************************************************************/
